@@ -1654,6 +1654,11 @@ class Building:
       ret = output_filename
     return ret
 
+  @staticmethod
+  def eliminate_duplicate_funcs(filename):
+    import duplicate_function_eliminator
+    duplicate_function_eliminator.eliminate_duplicate_funcs(filename)
+
   @staticmethod
   def closure_compiler(filename, pretty=True):
     if not check_closure_compiler():
